@@ -46,15 +46,15 @@ const_key!(RIGHT, Right);
 const_key!(TAB, Tab);
 const_key!(UP, Up);
 const_key!(PAGE_UP, PageUp);
-const_key!(CONTROL_A, Char('a'), KeyModifiers::CONTROL);
 const_key!(CONTROL_C, Char('c'), KeyModifiers::CONTROL);
-const_key!(CONTROL_V, Char('v'), KeyModifiers::CONTROL);
 const_key!(CONTROL_H, Char('h'), KeyModifiers::CONTROL);
+const_key!(CONTROL_F, Char('f'), KeyModifiers::CONTROL);
 const_key!(CONTROL_N, Char('n'), KeyModifiers::CONTROL);
 const_key!(CONTROL_O, Char('o'), KeyModifiers::CONTROL);
 const_key!(CONTROL_Q, Char('q'), KeyModifiers::CONTROL);
 const_key!(CONTROL_S, Char('s'), KeyModifiers::CONTROL);
 const_key!(CONTROL_U, Char('u'), KeyModifiers::CONTROL);
+const_key!(CONTROL_V, Char('v'), KeyModifiers::CONTROL);
 const_key!(CONTROL_X, Char('x'), KeyModifiers::CONTROL);
 const_key!(CONTROL_UP, Up, KeyModifiers::CONTROL);
 const_key!(CONTROL_DOWN, Down, KeyModifiers::CONTROL);
